// DeloneTriangulation.h

#ifndef _DELONE_TRIANGULATION_H
#define _DELONE_TRIANGULATION_H

#include <iostream>
#include <vector>
#include <ext/hash_set>
#include <set>
#include <list>
#include <NTL/mat_ZZ.h>

#include "Voronoi.h"
#include "Simplex.h"
#include "SimplexContainer.h"
#include "Facet.h"
#include "Regulator.h"

typedef __gnu_cxx::hash_set<Simplex*, SimplexHash, SimplexEq> hash_simplices;

class DeloneTriangulation {
 public:
  DeloneTriangulation();
  DeloneTriangulation(const DeloneTriangulation& Del);
  ~DeloneTriangulation();

  DeloneTriangulation& operator=(const DeloneTriangulation& Del);

  bool is_equivalent(DeloneTriangulation& Del);

  int compute_no_neighbours();
  DeloneTriangulation compute_nth_neighbour(const int n);
  // before calling compute_nth_neighbour you have to call compute_no_neighbours
  void compute_lower_bound(NTL::ZZ& invariant, double& lower_bound);

  void read_COOP_file(FILE* coop_file);
  void write_COOP_file(FILE* coop_file) const;

  friend DeloneTriangulation principal_domain();
  friend DeloneTriangulation barnes_trenerry_domain();

 private:
  hash_simplices simplices;
  int no_neighbours;

  std::set<Facet> facets; // facets of Delone triangulations
  std::set<Regulator> regulators; 
  std::vector<std::set<Regulator>::iterator> irredundants; // facets of secondary cone

  void make_neighbours();
  void destroy_neighbours();
  void make_facets_and_regulators();
  void destroy_facets_and_regulators();
  DeloneTriangulation do_flip(std::list<Facet*> flip_facets) const;

  static SimplexContainer all_simplices;
};

#endif




